require 'json'
require 'faraday'
require 'active_support/core_ext/class/attribute_accessors'

module KarrierWave
  class Client
    SANDBOX_HOST = 'http://sandbox-api1-kage.kakao.com'

    def self.for_config(config_name)
      @_karrierwave_clients ||= {}
      @_karrierwave_clients[config_name] ||= new(config_name: config_name)
    end

    attr_reader :config_name

    def initialize(options = {})
      options = options.dup

      @config_name = options.delete(:config_name)
    end

      # Upload
    def store(path, content_type)
      resp = connection.post "/up/#{self.config_name}/", {
        'file_1' => Faraday::UploadIO.new(path, content_type)
      }

      return JSON.parse(resp.body) if resp.success?
      raise KarrierWave::UploadError, 'failed to upload.'
    end

    # Download

    # Streaming Download

    # Status
    def status(access_key)
      resp = connection.get "/status/#{access_key}/"
      return JSON.parse(resp.body) if resp.success?
      raise KarrierWave::InvalidRequest
    end

    # Info
    def info(access_key)
      resp = connection.get "/info/#{access_key}/"
      return JSON.parse(resp.body) if resp.success?
      raise KarrierWave::InvalidRequest
    end

    def meta(access_key)
      resp = download_connection.get "/dn/#{access_key}/meta.json"
      return JSON.parse(resp.body) if resp.success?
      raise KarrierWave::InvalidRequest
    end

    # Copy

    # Remove
    def remove(access_key)
      resp = connection.get "/rm/#{access_key}/"
      return JSON.parse(resp.body) if resp.success?
      raise KarrierWave::InvalidRequest
    end

    def connection
      @connection ||= Faraday.new(url: KarrierWave.config.host) do |conn|
        conn.request :multipart
        conn.request :url_encoded
        conn.adapter :net_http
        conn.proxy KarrierWave.config.http_proxy
      end
    end

    def download_connection
      @download_connection ||= Faraday.new(url: KarrierWave.config.download_host) do |conn|
        conn.request :url_encoded
        conn.adapter :net_http
        conn.proxy KarrierWave.config.http_proxy
      end
    end
  end
end
