require 'karrierwave/client'

module KarrierWave
  class File
    attr_writer :content_type
    attr_reader :mounter, :file, :identifier, :filename, :config_name, :snapshot_filename

    def initialize(mounter, config_name, identifier: nil, filename: nil, snapshot_filename: nil, type: nil)
      @mounter = mounter
      @config_name = config_name
      @identifier = identifier
      @filename = filename
      @snapshot_filename = snapshot_filename
      @type = type
    end

    def store(new_file)
      @file = new_file
      resp = client.store(@file.path, @file.content_type)
      @identifier = resp['access_key']
      true
    end

    def filename
      @filename ||= identifier && info[0]['filename']
    end

    def content_type
      @content_type ||= identifier && info[0]['content-type']
    end

    def delete
      identifier && info && client.remove(identifier)
    end

    def info
      @info ||= client.info(identifier)
    end

    def meta
      @meta ||= client.meta(identifier)
    end

    def type
      @type ||= begin
        if identifier && meta['original']['ani']
          :animated
        else
          :image
        end
      end
    end

    # Public: url to source of media. Supports resizing only when type is TYPE_IMAGE.
    #
    # Returns url string.
    def url(version = nil)
      return nil if identifier.blank?

      if type.to_sym == :image && version
        "#{KarrierWave.config.download_host}/dn/#{identifier}/#{filename}?convert=#{version.to_s}"
      else
        "#{KarrierWave.config.download_host}/dn/#{identifier}/#{filename}"
      end
    end

    # Public: url to snapshot of source.
    #
    # Returns url string.
    def snapshot_url(version = nil)
      return nil if identifier.blank?

      "#{KarrierWave.config.download_host}/dn/#{identifier}/#{snapshot_filename || filename}#{version && "?convert=#{version.to_s}"}"
    end

    def to_json(options = {})
      {
        key: identifier,
        filename: filename,
        type: type
      }
    end

    private

    def client
      KarrierWave::Client.for_config(config_name)
    end
  end
end
