module KarrierWave
  class RemoteFile
    delegate :path, to: :file

    attr_reader :file

    def initialize(file)
      if file.is_a?(StringIO)
        @file = generate_tempfile_from(file)
      else
        @file = file
      end
    end

    def content_type
      MIME::Types.type_for(filename_from_header || filename_from_uri).first.to_s
    end

    def filename_from_header
      if @file.meta.include? 'content-disposition'
        match = @file.meta['content-disposition'].match(/filename="?([^"]+)/)
        match && !match[1].empty? && match[1]
      end
    end

    def filename_from_uri
      ::File.basename(@file.base_uri.path)
    end

    protected

    def generate_tempfile_from(string_io)
      Tempfile.new(['karrierwave', ::File.extname(string_io.base_uri.path)]).tap do |file|
        file.binmode
        OpenURI::Meta.init(file, string_io)
        file << string_io.string
        file.close
      end
    end
  end
end
